"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.encodeJSON = exports.validateStreamFormat = exports.isSupportedRawFormat = exports.isStreamableJSONFamily = exports.isNotStreamableJSONFamily = void 0;
const streamableJSONFormats = [
    'JSONEachRow',
    'JSONStringsEachRow',
    'JSONCompactEachRow',
    'JSONCompactStringsEachRow',
    'JSONCompactEachRowWithNames',
    'JSONCompactEachRowWithNamesAndTypes',
    'JSONCompactStringsEachRowWithNames',
    'JSONCompactStringsEachRowWithNamesAndTypes',
];
const recordsJSONFormats = ['JSONObjectEachRow'];
const singleDocumentJSONFormats = [
    'JSON',
    'JSONStrings',
    'JSONCompact',
    'JSONCompactStrings',
    'JSONColumnsWithMetadata',
];
const supportedJSONFormats = [
    ...recordsJSONFormats,
    ...singleDocumentJSONFormats,
    ...streamableJSONFormats,
];
const supportedRawFormats = [
    'CSV',
    'CSVWithNames',
    'CSVWithNamesAndTypes',
    'TabSeparated',
    'TabSeparatedRaw',
    'TabSeparatedWithNames',
    'TabSeparatedWithNamesAndTypes',
    'CustomSeparated',
    'CustomSeparatedWithNames',
    'CustomSeparatedWithNamesAndTypes',
    'Parquet',
];
const streamableFormat = [
    ...streamableJSONFormats,
    ...supportedRawFormats,
];
function isNotStreamableJSONFamily(format) {
    return (singleDocumentJSONFormats.includes(format) ||
        recordsJSONFormats.includes(format));
}
exports.isNotStreamableJSONFamily = isNotStreamableJSONFamily;
function isStreamableJSONFamily(format) {
    return streamableJSONFormats.includes(format);
}
exports.isStreamableJSONFamily = isStreamableJSONFamily;
function isSupportedRawFormat(dataFormat) {
    return supportedRawFormats.includes(dataFormat);
}
exports.isSupportedRawFormat = isSupportedRawFormat;
function validateStreamFormat(format) {
    if (!streamableFormat.includes(format)) {
        throw new Error(`${format} format is not streamable. Streamable formats: ${streamableFormat.join(',')}`);
    }
    return true;
}
exports.validateStreamFormat = validateStreamFormat;
/**
 * Encodes a single row of values into a string in a JSON format acceptable by ClickHouse.
 * @param value a single value to encode.
 * @param format One of the supported JSON formats: https://clickhouse.com/docs/en/interfaces/formats/
 * @returns string
 */
function encodeJSON(value, format) {
    if (supportedJSONFormats.includes(format)) {
        return JSON.stringify(value) + '\n';
    }
    throw new Error(`The client does not support JSON encoding in [${format}] format.`);
}
exports.encodeJSON = encodeJSON;
//# sourceMappingURL=formatter.js.map