"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.default = exports.CHANNELS = void 0;
var _notificationCatcherProvider = _interopRequireDefault(require("./providers/notificationCatcherProvider"));
var _sender = _interopRequireDefault(require("./sender"));
var _dedupe = _interopRequireDefault(require("./util/dedupe"));
var _logger = _interopRequireDefault(require("./util/logger"));
var _providers = _interopRequireDefault(require("./providers"));
var _providers2 = _interopRequireDefault(require("./strategies/providers"));
function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }
/* global $Keys */
// Types
const CHANNELS = exports.CHANNELS = {
  email: 'email',
  push: 'push',
  sms: 'sms',
  voice: 'voice',
  webpush: 'webpush',
  slack: 'slack'
};

// Defaults to fallback

class NotifmeSdk {
  logger = _logger.default;
  constructor(options) {
    const mergedOptions = this.mergeWithDefaultConfig(options);
    const providers = (0, _providers.default)(mergedOptions.channels);
    const strategies = (0, _providers2.default)(mergedOptions.channels);
    this.sender = new _sender.default((0, _dedupe.default)([...Object.keys(CHANNELS), ...Object.keys(providers)]), providers, strategies);
  }
  mergeWithDefaultConfig({
    channels,
    ...rest
  }) {
    return {
      useNotificationCatcher: false,
      ...rest,
      channels: rest.useNotificationCatcher ? _notificationCatcherProvider.default.getConfig(Object.keys(CHANNELS)) : {
        ...channels,
        email: {
          providers: [],
          multiProviderStrategy: 'fallback',
          ...(channels ? channels.email : null)
        },
        push: {
          providers: [],
          multiProviderStrategy: 'fallback',
          ...(channels ? channels.push : null)
        },
        sms: {
          providers: [],
          multiProviderStrategy: 'fallback',
          ...(channels ? channels.sms : null)
        },
        voice: {
          providers: [],
          multiProviderStrategy: 'fallback',
          ...(channels ? channels.voice : null)
        },
        webpush: {
          providers: [],
          multiProviderStrategy: 'fallback',
          ...(channels ? channels.webpush : null)
        },
        slack: {
          providers: [],
          multiProviderStrategy: 'fallback',
          ...(channels ? channels.slack : null)
        }
      }
    };
  }
  send(request) {
    return this.sender.send(request);
  }
}
exports.default = NotifmeSdk;