"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.default = factory;
var _logger = _interopRequireDefault(require("../logger"));
var _mailgun = _interopRequireDefault(require("./mailgun"));
var _mandrill = _interopRequireDefault(require("./mandrill"));
var _notificationCatcher = _interopRequireDefault(require("./notificationCatcher"));
var _sendgrid = _interopRequireDefault(require("./sendgrid"));
var _ses = _interopRequireDefault(require("./ses"));
var _sendmail = _interopRequireDefault(require("./sendmail"));
var _smtp = _interopRequireDefault(require("./smtp"));
var _sparkpost = _interopRequireDefault(require("./sparkpost"));
function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }
// Types
function factory({
  type,
  ...config
}) {
  switch (type) {
    // Development
    case 'logger':
      return new _logger.default(config, 'email');
    case 'notificationcatcher':
      return new _notificationCatcher.default('email');

    // Custom
    case 'custom':
      return config;

    // Protocols
    case 'sendmail':
      return new _sendmail.default(config);
    case 'smtp':
      return new _smtp.default(config);

    // Providers
    case 'mailgun':
      return new _mailgun.default(config);
    case 'mandrill':
      return new _mandrill.default(config);
    case 'sendgrid':
      return new _sendgrid.default(config);
    case 'ses':
      return new _ses.default(config);
    case 'sparkpost':
      return new _sparkpost.default(config);
    default:
      throw new Error(`Unknown email provider "${type}".`);
  }
}