"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.default = void 0;
var _crypto = _interopRequireDefault(require("crypto"));
var _request = _interopRequireDefault(require("../../util/request"));
function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }
// Types
class EmailSendGridProvider {
  id = 'email-sendgrid-provider';
  constructor(config) {
    this.apiKey = config.apiKey;
  }
  async send(request) {
    const {
      id,
      userId,
      from,
      replyTo,
      subject,
      html,
      text,
      headers,
      to,
      cc,
      bcc,
      attachments
    } = request.customize ? await request.customize(this.id, request) : request;
    const generatedId = id || _crypto.default.randomBytes(16).toString('hex');
    const response = await (0, _request.default)('https://api.sendgrid.com/v3/mail/send', {
      method: 'POST',
      headers: {
        Authorization: `Bearer ${this.apiKey}`,
        'Content-Type': 'application/json',
        'User-Agent': 'notifme-sdk/v1 (+https://github.com/notifme/notifme-sdk)'
      },
      body: JSON.stringify({
        personalizations: [{
          to: [{
            email: to
          }],
          ...(cc && cc.length > 0 ? {
            cc: cc.map(email => ({
              email
            }))
          } : null),
          ...(bcc && bcc.length > 0 ? {
            bcc: bcc.map(email => ({
              email
            }))
          } : null)
        }],
        from: {
          email: from
        },
        ...(replyTo ? {
          reply_to: {
            email: replyTo
          }
        } : null),
        subject,
        content: [...(text ? [{
          type: 'text/plain',
          value: text
        }] : []), ...(html ? [{
          type: 'text/html',
          value: html
        }] : [])],
        headers,
        custom_args: {
          id: generatedId,
          userId
        },
        ...(attachments && attachments.length > 0 ? {
          attachments: attachments.map(({
            contentType,
            filename,
            content
          }) => ({
            type: contentType,
            filename,
            content: (typeof content === 'string' ? Buffer.from(content) : content).toString('base64')
          }))
        } : null)
      })
    });
    if (response.ok) {
      return generatedId;
    } else {
      const responseBody = await response.json();
      const [firstError] = responseBody.errors;
      const message = Object.keys(firstError).map(key => `${key}: ${firstError[key]}`).join(', ');
      throw new Error(`${response.status} - ${message}`);
    }
  }
}
exports.default = EmailSendGridProvider;