"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.default = void 0;
var _v = _interopRequireDefault(require("../../util/aws/v4"));
var _crypto = require("../../util/crypto");
var _request = _interopRequireDefault(require("../../util/request"));
var _mailComposer = _interopRequireDefault(require("nodemailer/lib/mail-composer"));
var _querystring = _interopRequireDefault(require("querystring"));
function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }
// types
class EmailSesProvider {
  id = 'email-ses-provider';
  constructor({
    region,
    accessKeyId,
    secretAccessKey,
    sessionToken
  }) {
    this.credentials = {
      region,
      accessKeyId,
      secretAccessKey,
      sessionToken
    };
  }
  async send(request) {
    if (request.text && typeof request.text !== 'string' && !(request.text instanceof Buffer) && !(request.text instanceof Uint8Array)) {
      throw new Error('The "chunk" argument must be of type string or an instance of Buffer or Uint8Array.');
    }
    const {
      region
    } = this.credentials;
    const host = `email.${region}.amazonaws.com`;
    const raw = (await this.getRaw(request.customize ? await request.customize(this.id, request) : request)).toString('base64');
    const body = _querystring.default.stringify({
      Action: 'SendRawEmail',
      Version: '2010-12-01',
      'RawMessage.Data': raw
    });
    const apiRequest = {
      method: 'POST',
      path: '/',
      headers: {
        Host: host,
        'Content-Type': 'application/x-www-form-urlencoded; charset=utf-8',
        'X-Amz-Content-Sha256': (0, _crypto.sha256)(body, 'hex'),
        'User-Agent': 'notifme-sdk/v1 (+https://github.com/notifme/notifme-sdk)'
      },
      body,
      region
    };
    const signer = new _v.default(apiRequest, 'ses');
    signer.addAuthorization(this.credentials, new Date());
    const response = await (0, _request.default)(`https://${host}${apiRequest.path}`, apiRequest);
    const responseText = await response.text();
    if (response.ok && responseText.includes('<MessageId>')) {
      return responseText.match(/<MessageId>(.*)<\/MessageId>/)[1];
    } else {
      throw new Error(`${response.status} - ${responseText}`);
    }
  }
  async getRaw({
    customize,
    ...request
  }) {
    const email = new _mailComposer.default(request).compile();
    email.keepBcc = true;
    return email.build();
  }
}
exports.default = EmailSesProvider;