"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.default = void 0;
var _request = _interopRequireDefault(require("../../util/request"));
function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }
// Types
class SmsCallrProvider {
  id = 'sms-callr-provider';
  constructor({
    login,
    password
  }) {
    this.apiKey = Buffer.from(`${login}:${password}`).toString('base64');
  }

  /*
   * Note: 'from', 'messageClass', 'ttl' are not supported.
   */
  async send(request) {
    const {
      id,
      userId,
      from,
      to,
      text,
      type,
      nature
    } = request.customize ? await request.customize(this.id, request) : request;
    const response = await (0, _request.default)('https://api.callr.com/rest/v1.1/sms', {
      method: 'POST',
      headers: {
        'Content-Type': 'application/json',
        Authorization: `Basic ${this.apiKey}`,
        'User-Agent': 'notifme-sdk/v1 (+https://github.com/notifme/notifme-sdk)'
      },
      body: JSON.stringify({
        from,
        to,
        body: text,
        options: {
          force_encoding: type === 'unicode' ? 'UNICODE' : 'GSM',
          nature: nature === 'marketing' ? 'MARKETING' : 'ALERTING',
          ...(userId || id ? {
            user_data: userId || id
          } : null)
        }
      })
    });
    const responseBody = await response.json();
    if (response.ok) {
      return responseBody.data;
    } else {
      const error = responseBody.data;
      throw new Error(Object.keys(error).map(key => `${key}: ${error[key]}`).join(', '));
    }
  }
}
exports.default = SmsCallrProvider;