"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.default = void 0;
var _request = _interopRequireDefault(require("../../util/request"));
function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }
// Types
class SmsClickatellProvider {
  id = 'sms-clickatell-provider';
  constructor(config) {
    // One-way integration API key
    this.apiKey = config.apiKey;
  }

  /*
   * Note: 'from', 'nature', 'messageClass' are not supported.
   */
  async send(request) {
    const {
      id,
      to,
      text,
      type,
      ttl
    } = request.customize ? await request.customize(this.id, request) : request;
    const response = await (0, _request.default)('https://platform.clickatell.com/messages', {
      method: 'POST',
      headers: {
        'Content-Type': 'application/json',
        Authorization: this.apiKey,
        'User-Agent': 'notifme-sdk/v1 (+https://github.com/notifme/notifme-sdk)'
      },
      body: JSON.stringify({
        // no `from` for one-way integrations
        to: [to],
        content: text,
        charset: type === 'unicode' ? 'UCS2-BE' : 'UTF-8',
        ...(ttl ? {
          validityPeriod: ttl
        } : null),
        ...(id ? {
          clientMessageId: id
        } : null)
      })
    });
    if (response.ok) {
      const responseBody = await response.json();
      if (responseBody.error) {
        throw new Error(responseBody.error);
      } else {
        return responseBody.messages[0].apiMessageId;
      }
    } else {
      throw new Error(await response.text());
    }
  }
}
exports.default = SmsClickatellProvider;