"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.default = void 0;
var _request = _interopRequireDefault(require("../../util/request"));
function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }
// Types
class SmsNexmoProvider {
  id = 'sms-nexmo-provider';
  constructor(config) {
    this.credentials = {
      api_key: config.apiKey,
      api_secret: config.apiSecret
    };
  }

  /*
   * Note: 'nature' is not supported.
   */
  async send(request) {
    const {
      from,
      to,
      text,
      type,
      ttl,
      messageClass
    } = request.customize ? await request.customize(this.id, request) : request;
    const response = await (0, _request.default)('https://rest.nexmo.com/sms/json', {
      method: 'POST',
      headers: {
        'Content-Type': 'application/json',
        'User-Agent': 'notifme-sdk/v1 (+https://github.com/notifme/notifme-sdk)'
      },
      body: JSON.stringify({
        ...this.credentials,
        from,
        to,
        text,
        type,
        ttl,
        'message-class': messageClass
      })
    });
    if (response.ok) {
      const responseBody = await response.json();
      const message = responseBody.messages[0];

      // Nexmo always returns 200 even for error
      if (message.status !== '0') {
        throw new Error(`status: ${message.status}, error: ${message['error-text']}`);
      } else {
        return message['message-id'];
      }
    } else {
      throw new Error(response.status);
    }
  }
}
exports.default = SmsNexmoProvider;