"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.default = void 0;
var _request = _interopRequireDefault(require("../../util/request"));
function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }
// Types
class SmsPlivoProvider {
  id = 'sms-plivo-provider';
  constructor({
    authId,
    authToken
  }) {
    this.authId = authId;
    this.apiKey = Buffer.from(`${authId}:${authToken}`).toString('base64');
  }

  /*
   * Note: 'type', 'nature', 'ttl', 'messageClass' are not supported.
   */
  async send(request) {
    const {
      from,
      to,
      text
    } = request.customize ? await request.customize(this.id, request) : request;
    const response = await (0, _request.default)(`https://api.plivo.com/v1/Account/${this.authId}/Message/`, {
      method: 'POST',
      headers: {
        Authorization: `Basic ${this.apiKey}`,
        'Content-Type': 'application/json',
        'User-Agent': 'notifme-sdk/v1 (+https://github.com/notifme/notifme-sdk)'
      },
      body: JSON.stringify({
        src: from,
        dst: to,
        text
      })
    });
    if (response.ok) {
      const responseBody = await response.json();
      return responseBody.message_uuid[0];
    } else {
      throw new Error(response.status === 401 ? await response.text() : (await response.json()).error);
    }
  }
}
exports.default = SmsPlivoProvider;