"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.default = void 0;
var _request = _interopRequireDefault(require("../../util/request"));
var _formData = _interopRequireDefault(require("form-data"));
function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }
// Types
class VoiceTwilioProvider {
  id = 'voice-twilio-provider';
  constructor({
    accountSid,
    authToken
  }) {
    this.accountSid = accountSid;
    this.apiKey = Buffer.from(`${accountSid}:${authToken}`).toString('base64');
  }
  async send(request) {
    const {
      from,
      to,
      url,
      method,
      fallbackUrl,
      fallbackMethod,
      statusCallback,
      statusCallbackEvent,
      sendDigits,
      machineDetection,
      machineDetectionTimeout,
      timeout
    } = request.customize ? await request.customize(this.id, request) : request;
    const form = new _formData.default();
    form.append('From', from);
    form.append('To', to);
    form.append('Url', url);
    if (method) form.append('Method', method);
    if (fallbackUrl) form.append('FallbackUrl', fallbackUrl);
    if (fallbackMethod) form.append('FallbackMethod', fallbackMethod);
    if (statusCallback) form.append('StatusCallback', statusCallback);
    if (statusCallbackEvent) {
      statusCallbackEvent.forEach(event => form.append('StatusCallbackEvent', event));
    }
    if (sendDigits) form.append('SendDigits', sendDigits);
    if (machineDetection) form.append('MachineDetection', machineDetection);
    if (machineDetectionTimeout) form.append('MachineDetectionTimeout', machineDetectionTimeout);
    if (timeout) form.append('Timeout', timeout);
    const response = await (0, _request.default)(`https://api.twilio.com/2010-04-01/Accounts/${this.accountSid}/Calls.json`, {
      method: 'POST',
      headers: {
        Authorization: `Basic ${this.apiKey}`,
        'User-Agent': 'notifme-sdk/v1 (+https://github.com/notifme/notifme-sdk)'
      },
      body: form
    });
    const responseBody = await response.json();
    if (response.ok) {
      return responseBody.sid;
    } else {
      throw new Error(`${response.status} - ${responseBody.message}`);
    }
  }
}
exports.default = VoiceTwilioProvider;