"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.default = void 0;
var _logger = _interopRequireDefault(require("./util/logger"));
var _logger2 = _interopRequireDefault(require("./providers/logger"));
var _registry = _interopRequireDefault(require("./util/registry"));
function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }
// Types
class Sender {
  constructor(channels, providers, strategies) {
    this.channels = channels;
    this.providers = providers;
    this.strategies = strategies;

    // note : we can do this memoization because we do not allow to add new provider
    this.senders = Object.keys(strategies).reduce((acc, channel) => {
      acc[channel] = this.providers[channel].length > 0 ? strategies[channel](this.providers[channel]) : async request => {
        _logger.default.warn(`No provider registered for channel "${channel}". Using logger.`);
        const provider = _registry.default.getInstance(`${channel}-logger-default`, () => new _logger2.default({}, channel));
        return {
          success: true,
          channel,
          providerId: provider.id,
          id: await provider.send(request)
        };
      };
      return acc;
    }, {});
  }
  async send(request) {
    const resultsByChannel = await this.sendOnEachChannel(request);
    const result = resultsByChannel.reduce((acc, {
      success,
      channel,
      providerId,
      ...rest
    }) => ({
      ...acc,
      channels: {
        ...(acc.channels || null),
        [channel]: {
          id: rest.id,
          providerId
        }
      },
      ...(!success ? {
        status: 'error',
        errors: {
          ...(acc.errors || null),
          [channel]: rest.error.message
        }
      } : null)
    }), {
      status: 'success'
    });
    return result;
  }
  async sendOnEachChannel(request) {
    return Promise.all(Object.keys(request).filter(channel => this.channels.includes(channel)).map(async channel => {
      try {
        return {
          success: true,
          channel,
          ...(await this.senders[channel]({
            ...request.metadata,
            ...request[channel]
          }))
        };
      } catch (error) {
        return {
          channel,
          success: false,
          error: error,
          providerId: error.providerId
        };
      }
    }));
  }
}
exports.default = Sender;