"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.default = factory;
var _fallback = _interopRequireDefault(require("./fallback"));
var _noFallback = _interopRequireDefault(require("./no-fallback"));
var _roundrobin = _interopRequireDefault(require("./roundrobin"));
function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }
// Types
const providerStrategies = {
  fallback: _fallback.default,
  'no-fallback': _noFallback.default,
  roundrobin: _roundrobin.default
};
const strategies = Object.keys(providerStrategies);
function factory(channels) {
  return Object.keys(channels).reduce((acc, key) => {
    const optionStrategy = channels[key].multiProviderStrategy;
    if (typeof optionStrategy === 'function') {
      acc[key] = optionStrategy;
    } else if (strategies.includes(optionStrategy)) {
      acc[key] = providerStrategies[optionStrategy];
    } else {
      throw new Error(`"${optionStrategy}" is not a valid strategy. Strategy must be a function or ${strategies.join('|')}.`);
    }
    return acc;
  }, {});
}